/*  This file is part of DropGather.

  DropGather is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  DropGather is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with DropGather.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "NIDAQmx.h"
#include "errorhandlers.h"

#ifndef DATACOLLECTION
#define DATACOLLECTION

class DataCollection
{
	int nChans;
	int nSamples;
	float64 * data;
	DataCollection() {};
public:
	DataCollection(int _nChans, int _nSamples) : nChans(_nChans), nSamples(_nSamples)
	{
		data = new float64[nSamples*nChans];
	};

	~DataCollection()
	{
		delete [] data;
	};
    
	operator float64*()
	{
		return data;
	}

	float64 ReadVal(int Chan, int Sample)
	{
		return data[Chan*nSamples+Sample];
	}

	void GetChan(int Chan, float * Samples, int _nSamples)
	{
		if(Chan >= nChans)
			throw Excpt(0, "Index channel above");

		if(nSamples < _nSamples)
			_nSamples = nSamples;

		for(int i = 0; i < _nSamples; i++)
			Samples[i] = (float) data[Chan*nSamples + i];
	}

	void GetChan(int Chan, float64 * Samples, int _nSamples)
	{
		if(Chan >= nChans)
			throw Excpt(0, "Index channel above");

		if(nSamples < _nSamples)
			_nSamples = nSamples;

		for(int i = 0; i < _nSamples; i++)
			Samples[i] = data[Chan*nSamples + i];
	}
};

#endif