/*  This file is part of DropGather.

  DropGather is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  DropGather is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with DropGather.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "imeProject.h"

imeProject::imeProject(int _nChans, int _nSamples) : nSamples(_nSamples), nChans(_nChans), data(nChans, nSamples)
{
	chan = new float[_nSamples];
	tarray = new float[_nSamples];
	for(int i = 0; i < _nSamples; i++)
		tarray[i] = (float) i;

	db.open("index.ini",std::ios_base::in);
	if(db.is_open())
	{
		db >> drop;
		if(db.fail() && !db.eof())
			throw Excpt(0, "Index not right");
	}
	else
        drop = 0;
	db.close();

	/*********************************************/
	// DAQmx Configure Code
	/*********************************************/
		// analog channels used are:
		// ai0		trigger					box1
		// ai1-7	chips 1-8 (8 of them)	box1
		// ai16		signal voltage (5v)		box2 - ai0
		// ai17-23	chips 9-15 (7 of them)	box2 - ai1-7
		// ai24		signal voltage (5v)		box2 - ai8
		// the signal voltage is connected to two inputs, only one needs to be used
		// software assumes ai24 is used right now.. output formats have to change to use ai16
	DAQError (DAQmxCreateAIVoltageChan(
		taskHandle,		// task to append too
		"Dev1/ai0:8,Dev1/ai17:24",	// Channels to append
		"",				// Channel names, defaults to ai0 ai1 ... ai16
		DAQmx_Val_RSE,	// Referenced single ended
		0.0,			// Minimum voltage to see
		10.0,			// Maximum voltage
		DAQmx_Val_Volts,// Scale to use
		NULL));			// Name of scale if custom

	DAQError (DAQmxCfgSampClkTiming(
		taskHandle,				// task to append too
		NULL,					// Timing mechinism
		50000.0,				// Samples per second
		DAQmx_Val_Rising,		// Edge to collect samples
		DAQmx_Val_FiniteSamps,	// Type of collection
		nSamples));				// Ammount of samples to collect

	DAQError (DAQmxCfgAnlgEdgeStartTrig(
		taskHandle,				// task to append too
		"Dev1/ai0",			// Name of channel to trigger on
		DAQmx_Val_RisingSlope,		// Edge to detect
		1.5));					// Trigger level
	DAQError (DAQmxSetAnlgEdgeStartTrigHyst(taskHandle, 0.1));
};

void imeProject::doCase(bool fail1, bool PbAg, bool Fill, bool Finish, int index)
{
	std::string pdffile;
	proc.str("");

	proc << drop << "_";

	if(fail1)
		proc << "1Fail_";

	if(PbAg)
		proc << "Pb_";
	else
		proc << "Ag_";

	if(Fill)
		proc << "UF_";
	else
		proc << "NoUF_";

	if(Finish)
		proc << "OSP_";
	else
		proc << "ENIG_";

	proc << "board_";
	proc << index;
	pdffile = proc.str();
	proc << ".txt";

	datafile.open(proc.str().c_str(),std::ios_base::out & std::ios_base::ate);

	if(datafile.is_open())
	{
		throw Excpt(0, "File already exists!");
	}
	else
	{
		datafile.close();
		datafile.clear();
		datafile.open(proc.str().c_str(),std::ios_base::out);
	}

	if(!datafile.is_open())
		throw Excpt(0, "File couldn't be created");

	/*********************************************/
	// DAQmx Start Code
	/*********************************************/
	DAQError (DAQmxStartTask(taskHandle));

	std::cout << " Ready... Waiting for trigger" << std::endl;

	/*********************************************/
	// DAQmx Read Code
	/*********************************************/
	DAQError (DAQmxReadAnalogF64(
		taskHandle,					// task to read from
		DAQmx_Val_Auto,				// How many samples to read
		DAQmx_Val_WaitInfinitely,	// How long to wait for data
		DAQmx_Val_GroupByChannel,	// How to fill array
		data,						// Array to be filled
		nChans*nSamples,			// Size of array to be filled
		&read,						// How many samples read
		NULL));						// Reserved, pass NULL, unknown function

	std::cout << read << " samples read." << std::endl;

	DAQError(DAQmxStopTask(taskHandle));


/*********************************/
// graphical display begins
	metafl("xwin");
    setpag("da4l");
	winsiz(800,600);

// graph trigger and rail voltage together
	disini();
	pagera();
	hwfont();
	axspos(450,1800);
	axslen(2200,1200);

	name("Time (0.02 msec)","x");
	name("Voltage","y");

	labdig(-1,"x");
	titlin("Data",1);
	graf(0.f, (float)nSamples, 0.f, 15000.f, -.5f, 10.f, 0.f, 1.f);
	title();
	
	char legendb[160];
	legini(legendb, 2, 20);
	leglin(legendb, "Trigger", 1);
	leglin(legendb, "5v Source", 2);

	color("magenta");
	// trigger data
	data.GetChan(0, chan, read);
	curve(tarray,chan,read);
	color("cyan");
	// rail voltage data
	data.GetChan(16, chan, read);
	curve(tarray,chan,read);

	color("fore");
	dash();
	xaxgit();
	legend(legendb, 7);
	disfin();
	
	// graph separate data channels, all 15 in their own window
	for (int ndx = 1; ndx < 16; ndx++)
	{
		showCurveWnd(ndx);
	}


/*********************************/
// PDF output file begins
	std::cout << "Writing PDF...";

/* buffer standard out */
//   std::cout.

	pdffile += ".pdf";
	char * file;
	file = new char[pdffile.size()+1];
	strcpy(file,pdffile.c_str());
    setfil(file);
	metafl("pdf");
	
    //setpag("da4l");
	disini();

	char *titles[] = {"Trigger","Chip 1","Chip 2","Chip 3","Chip 4","Chip 5","Chip 6","Chip 7","Chip 8","Chip 9","Chip 10","Chip 11","Chip 12","Chip 13","Chip 14","Chip 15","5v"};
	
	// graph trigger separately due to range issue
	int i = 0;
	pagera();
	hwfont();
	axspos(450,1800);
	axslen(2200,1200);

	name("Time (0.02 msec)","x");
	name("Voltage","y");

	labdig(-1,"x");

	titlin(titles[i],1);

	graf(0.f, (float)nSamples, 0.f, 15000.f, -.5f, 10.f, 0.f, 1.f);
	// x-lower x-upper first-x x-step y-lower y-upper first-y y-step
	title();

    color("red");
	data.GetChan(i, chan, read);
	curve(tarray,chan,read);

	color("fore");
	dash();
	xaxgit();

	endgrf();
	newpag();

	// continue graphing all other channels into the PDF
	for(int i = 1; i < nChans; i++)
	{
		pagera();
		hwfont();
		axspos(450,1800);
		axslen(2200,1200);

		name("Time (0.02 msec)","x");
		name("Voltage","y");

		labdig(-1,"x");

		titlin(titles[i],1);

		graf(0.f, (float)nSamples, 0.f, 15000.f, -.5f, 5.5f, 0.f, 1.f);
		// x-lower x-upper first-x x-step y-lower y-upper first-y y-step
		title();

        color("red");
		data.GetChan(i, chan, read);
		curve(tarray,chan,read);

		color("fore");
		dash();
		xaxgit();

		endgrf();
		if(i != nChans-1)
		{
			newpag();
		}
	}
	disfin();
	delete [] file;

	std::cout << "Done" << std::endl;


/*********************************/
// text data file output begins
//	std::cout << "List any new damage to board or any components that have failed" << std::endl;
//	std::cout << "Limit your response to less than 20 lines" << std::endl;
//	std::cout << "Type 'eof' followed by the <enter> key to end text entry" << std::endl << std::endl;
	std::vector<std::string> lines;
	std::string oneline = "";

	// removed commenting block to speedup working
/*
	while(oneline != "eof" && lines.size() < 21)
	{
		std::getline(std::cin, oneline);
		lines.push_back(oneline);
	}
*/
	std::cout << "Writing to file..." << std::endl;

	datafile << "usr" << std::endl;
/*
	for(size_t i = 0; i < lines.size()-1; i++)
	{
		datafile << lines[i] << std::endl;
	}
*/
	// changed index from lines.size()-1 to this as it is, removed comment handling loop above
/*
	for(size_t i = lines.size(); i < 20; i++)
	{
		datafile << std::endl;
	}
*/
	datafile << read << " " << nChans << std::endl;

	datafile << "Trigger\tChip 1\tChip 2\tChip 3\tChip 4\tChip 5\tChip 6\tChip 7\tChip 8\tChip 9\tChip 10\tChip 11\tChip 12\tChip 13\tChip 14\tChip 15\t5v" << std::endl;

	datafile.precision(6);
	datafile.fill(0);
	datafile.width(8);
	datafile.setf(datafile.fixed|datafile.left);

    for(int i = 0; i < nSamples; i++)
	{
		datafile << data.ReadVal(0, i);
		for(int j = 1; j < nChans; j++)
			datafile <<  "\t" << data.ReadVal(j, i);
		datafile << "\n";
	}

	// Get user description
	if(datafile.eof())
		throw Excpt(0, "File error!");

	datafile.close();

	std::cout << "Drop index " << drop << " completed." << std::endl;

	drop++;

	// attempt to dump drop count to index file every time
	// no warning on fail here, leave file output failure until program exit
	db.clear();
	db.open("index.ini",std::ios_base::out);
	if(db.is_open())
	{
		db << drop;
		db.close();
	}
};

imeProject::~imeProject()
{
	delete [] tarray;
	delete [] chan;
	db.clear();
	db.open("index.ini",std::ios_base::out);
	if(db.is_open())
	{
		db << drop;
		if(db.fail() && !db.eof())
			throw Excpt(0, "Index not written check");
	}
};

// shows a plot in window for the data indicated by chan
// input:
// part - string, the part number or other label
// index - int, channel index for the call to GetChan()
void imeProject::showCurveWnd(const int index)
{
	disini();
	pagera();
	hwfont();
	axspos(450,1800);
	axslen(2200,1200);

	name("Time (0.02 msec)","x");
	name("Voltage","y");

	labdig(-1,"x");
	titlin("Data",1);
	graf(0.f, (float)nSamples, 0.f, 15000.f, -.5f, 5.5f, 0.f, 1.f);
	title();

	char legendb[160];
	char legtext[10];
	sprintf(legtext, "Part %d\0", index);
	legini(legendb, 1, 20);
	leglin(legendb, legtext, 1);
	char* colors[7] = {"green", "yellow", "magenta", "cyan", "red", "blue", "orange"};
	color(colors[index%2]); // alternate green and yellow
	data.GetChan(index, chan, read);
	curve(tarray,chan,read);

	color("fore");
	dash();
	xaxgit();
	legend(legendb, 7); // display upper right
	disfin();
}
